#include "common.h"

#include "common.c"

int main(int ac,char **av)
{
    __u32 flowlabel = 0;
    int sock, len, err_num, acpt;
    socklen_t sin6_len = 0;
    uint16_t port = 0;
    struct in6_addr addr;
    struct sockaddr_in6 sin6, d_sin6;
    char *target, v6addr[128];

    sock = len = err_num = acpt = 0;
    if (ac != 4) {
	printf("usage: %s <flowlabel> <bind_host> <wait_port>\n",av[0]);
	return 0;
    }

    sscanf(av[1],"%x",&flowlabel);
    if (flowlabel == 0) {
	printf("flowlabel is null.\n");
	return -1;
    }
    target = av[2];
    sscanf(av[3],"%d",&acpt);
    if (acpt == 0) {
	printf("dst_port is null.\n");
	return -2;
    } else {
	port = (uint16_t)acpt;
	acpt = 0;
    }

    if (getip6name(&addr, target, "bind_addr") < 0) return -3;
    sin6_len = sizeof(sin6);
    memset(&sin6, 0, sin6_len);
    sock = socket(PF_INET6, SOCK_STREAM, IPPROTO_TCP);
    if (sock < 0) {
	printf("cannot open INET6 socket.\n");
	return -4;
    }

    sin6.sin6_family = AF_INET6;
    memcpy(&sin6.sin6_addr, &addr, 16);
    if (getsockname(sock, (struct sockaddr*)&sin6, &sin6_len) == -1) {
	perror("getsockname");
	return -20;
    }
    sin6.sin6_port = htons(port);

    if (bind(sock, (struct sockaddr*)&sin6, sin6_len) == -1) {
	perror("bind");
	err_num = -10;
	goto err;
    }
    if (listen(sock, 2) == -1) {
	perror("listen");
	err_num = -11;
	goto err;
    }

//    while (1) {
    memset(&d_sin6, 0, sizeof(d_sin6));
    acpt = accept(sock, (struct sockaddr *)&d_sin6, &sin6_len);
    memcpy(&addr, &d_sin6.sin6_addr, 16); memset(v6addr, 0, 128);
    if (inet_ntop(AF_INET6, &addr, v6addr, 128) == NULL) {
	perror("inet_ntop");
	memcpy(v6addr, target, (strlen(target) < 127 ? strlen(target) : 127));
    }
    printf("dst_addr = %s\n",v6addr);

    if (set_rxflow(acpt) < 0) {
	printf("can't get flowlabel\n");
    }
    memset(v6addr, 0, 128);
    len = recv(acpt, v6addr, 128, 0);
    printf("message: %s\n", v6addr);
    if (getlabel(acpt, &d_sin6, &sin6_len) < 0) {
	perror("getlabel");
	return -21;
    }
    printf("dst_host flow has 0x%x.\n",ntohl(d_sin6.sin6_flowinfo));

    if (set_flowlabel(acpt, &d_sin6, flowlabel) < 0) {
	printf("cannot use flowlabel.\n");
	close(acpt);
	err_num = -5;
	goto err;
    }
    if (setlabel(acpt, htonl(flowlabel)) < 0) {
	perror("setlabel");
	release_flowlabel(acpt, &d_sin6, flowlabel);
	close(acpt);
	err_num = -5;
	goto err;
    }

    len = 0; memset(v6addr, 0, 128);
    sprintf(v6addr, "flowlabel = 0x%x", flowlabel);
    len = send(acpt, v6addr, strlen(v6addr), 0);
    if (len < 0) {
	perror("send");
	err_num = -7;
	release_flowlabel(acpt, &d_sin6, flowlabel);
	close(acpt);
	goto err;
    }
    len = recv(acpt, v6addr, 128, 0);
    release_flowlabel(acpt, &d_sin6, flowlabel);
    close(acpt);
//   }

 err:
    close(sock);
    return err_num;
}
